package com.consultingwerk.plugins.smartunitlaunchconfig.shortcuts;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationType;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.core.Launch;
import org.eclipse.debug.core.model.ILaunchConfigurationDelegate;

import com.consultingwerk.plugins.common.utilities.ConsoleHelper;
import com.consultingwerk.plugins.common.utilities.FileHelper;
import com.consultingwerk.plugins.common.utilities.PreferencesHelper;
import com.consultingwerk.plugins.smartunitlaunchconfig.SmartUnitAttributes;
import com.consultingwerk.plugins.smartunitlaunchconfig.delegates.ScenarioDelegate;
import com.consultingwerk.plugins.smartunitlaunchconfig.delegates.TestClassDelegate;

/**
 * Shortcut to launch a SmartUnit test
 */
public abstract class SmartUnitShortcut 
{
    protected final ILaunchManager manager;
    
    /**
     * Constructor for the SmartUnitShortcut class
     */
    protected SmartUnitShortcut ()
    {
        this.manager = DebugPlugin.getDefault().getLaunchManager();
    }
    
    /**
     * Creates a launch config and launches it via delegate
     * @param filename The name of the file to launch
     * @param method The method in the gile to launch
     * @param typeID The ID of the LaunchConfigurationType
     * @param mode The mode to launch in
     */
    protected void launch (String filename,
                           String method,
                           String typeID,
                           String mode)
    {
        ILaunchConfigurationType type;
        ILaunchConfigurationDelegate delegate;
        ILaunch launch;
        
        try
        {
            type = this.manager.getLaunchConfigurationType(typeID);

            ILaunchConfigurationWorkingCopy config = type.newInstance(null, this.getUniqueLaunchConfigName(filename, method, type));
            
            config.setAttribute (SmartUnitAttributes.BuildFile,    PreferencesHelper.getStringPreference (SmartUnitAttributes.Store, SmartUnitAttributes.BuildFile));
            config.setAttribute (SmartUnitAttributes.ANT,          PreferencesHelper.getStringPreference (SmartUnitAttributes.Store, SmartUnitAttributes.ANT));
            config.setAttribute (SmartUnitAttributes.Output,       PreferencesHelper.getStringPreference (SmartUnitAttributes.Store, SmartUnitAttributes.Output));
            config.setAttribute (SmartUnitAttributes.TestFile,     filename);
            config.setAttribute (SmartUnitAttributes.TestMethod,   method);
            config.setAttribute (SmartUnitAttributes.AntArguments, PreferencesHelper.getStringPreference (SmartUnitAttributes.Store, SmartUnitAttributes.AntArguments));
            config.setAttribute (SmartUnitAttributes.Services,     PreferencesHelper.getStringPreference (SmartUnitAttributes.Store, SmartUnitAttributes.Services));
            config.setAttribute (SmartUnitAttributes.UseConsole,   PreferencesHelper.getBoolPreference   (SmartUnitAttributes.Store, SmartUnitAttributes.UseConsole));
            config.setAttribute (SmartUnitAttributes.UseProfiler,  PreferencesHelper.getBoolPreference   (SmartUnitAttributes.Store, SmartUnitAttributes.UseProfiler));
            
            if(filename.endsWith(".cls"))
                config.setAttribute (SmartUnitAttributes.TestClassTarget, PreferencesHelper.getStringPreference (SmartUnitAttributes.Store, SmartUnitAttributes.TestClassTarget));
            else if(filename.endsWith(".scenario"))
                config.setAttribute (SmartUnitAttributes.ScenarioTarget,  PreferencesHelper.getStringPreference (SmartUnitAttributes.Store, SmartUnitAttributes.ScenarioTarget));
            
            config.doSave();
            
            delegate = this.getDelegateForTypeId(typeID);
            launch = new Launch(config, mode, null);
            
            manager.addLaunch(launch);
            delegate.launch(config, mode, launch, null);
            manager.removeLaunch(launch);
        }
        catch (CoreException e)
        {
            ConsoleHelper.printException(e);
        }
    }
    
    /**
     * Returns a unique name for the file in the given type
     * @param filename The file to launch
     * @param method The method in the file to launch
     * @param type The LaunchConfigurationType
     * @return A unique name for a launch of the file in the launchconfig
     * @throws CoreException 
     */
    protected String getUniqueLaunchConfigName (String filename, String method, ILaunchConfigurationType type) throws CoreException
    {
        for(ILaunchConfiguration config: this.manager.getLaunchConfigurations(type))
        {
            if(   config.getAttribute(SmartUnitAttributes.TestFile, "").equals(filename)
               && (   method == null || method.equals("")
                   || config.getAttribute(SmartUnitAttributes.TestMethod, "").equals(method)))
               return config.getName();
        }
        if(method == null || method.equals(""))
            return this.manager.generateLaunchConfigurationName(FileHelper.shortFilename(filename));
        else
            return this.manager.generateLaunchConfigurationName(FileHelper.shortFilename(filename) + "_" + method);
    }
    
    /**
     * Returns a delegate for a given LaunchConfigurationType
     * @param typeID The LaunchConfigurationTypes ID
     * @return The LaunchConfigurationTypes delegate
     */
    protected ILaunchConfigurationDelegate getDelegateForTypeId (String typeID)
    {
        switch(typeID)
        {
            case SmartUnitAttributes.TestClassTypeID:
                return new TestClassDelegate();
            case SmartUnitAttributes.ScenarioTestTypeID:
                return new ScenarioDelegate();
            default:
                return null;
        }
    }
}
